<?php
///////////////////////////////////////////////////////////
// day-mysql.php.inc
///////////////////////////////////////////////////////////
//
// An implementation of the day class with MySQL syntax.
//
// Version:    1.1
// Written by: Justin Zipkin 
//             justin.zipkin@db.com
//
// Modified:   07.11.2005 (jz)
//             12.11.2005 (jz) - fixed hyphen issue
//
///////////////////////////////////////////////////////////

define("SUNDAY",0);
define("MONDAY",1);
define("TUESDAY",2);
define("WEDNESDAY",3);
define("THURSDAY",4);
define("FRIDAY",5);
define("SATURDAY",6);

class day {
  private $db;
  private $date, $day, $month, $year, $dow;
  private $tasks;
  
  #########################################################
  # PUBLIC FUNCTIONS
  #########################################################
  
  /////////////////////////////////////////////////////////
  // Cunstructor ($day, $month, $year)
  //
  // Takes:   A date as three numeric strings
  // Returns: Nothing
  //
  // Sets up the class private variables $day, $month, 
  // $year, and $dow (Day of week).
  //
  /////////////////////////////////////////////////////////
  function __construct($date) {
    if (!class_exists('db')) {
      die('Error:  No database class included!  Please refer to documentation.');
    }
    
    $this->date=$date;
    $this->day=date('d', $date);
    $this->month=date('m', $date);
    $this->year=date('Y', $date);
    $this->dow=date('w',mktime(0,0,0,$this->month,$this->day,$this->year));
    $this->loadTasks();
  }
  
  /////////////////////////////////////////////////////////
  // getTasks()
  //
  // Takes:   Nothing
  // Returns: An array of objects
  //
  // This function will return an array of objects based on
  // the tasks configured to run on a particular day.
  // Objects can be accessed via the SQL column name (i.e.
  // getTasks()->filename).
  //
  /////////////////////////////////////////////////////////
  public function getTasks() {
    return $this->tasks;
  }
  
  /////////////////////////////////////////////////////////
  // getDistribution()
  //
  // Takes:   Nothing
  // Returns: An array of objects
  //
  // This function will return an array of objects based on
  // the distribution of tasks configured to run on a 
  // particular day.
  //
  /////////////////////////////////////////////////////////
  public function getDistribution() {
    $sqloptions=implode(' OR ',$this->getSQLOptions());
    $sql="SELECT type, COUNT(type) AS typecount FROM tasks WHERE $sqloptions GROUP BY type";
    
    $distribution=$this->db->dosql($sql);

    return $distribution;
  }
  
  /////////////////////////////////////////////////////////
  // getDateStamp()
  //
  // Takes:   Nothing
  // Returns: A unix timestamp
  //
  // This function will return a unix timestamp for the
  // set date.
  //
  /////////////////////////////////////////////////////////
  public function getDateStamp() {
    return mktime(0,0,0,$this->month,$this->day,$this->year);
  }

  /////////////////////////////////////////////////////////
  // drawDistribution()
  //
  // Takes:   Nothing
  // Returns: Nothing
  //
  // Draw a table explaining the task distribution for the
  // current object.
  //
  /////////////////////////////////////////////////////////
  public function drawDistribution() {
    $distribution=$this->getDistribution();

    print "<table cellpadding=\"0\" cellspacing=\"1\" width=\"100%\" border=\"0\" align=\"center\">\n";
    print "<tr><td colspan=2><h3 align=\"center\">Viewing " . count($this->tasks) . " tasks on " . date('l, j M Y',mktime(0,0,0,$this->month,$this->day,$this->year)) . "</h3></td></tr>\n";
    print "<tr><td colspan=2><hr /></td></tr>\n";
    print "<tr><td align=\"center\" colspan=\"2\"><font size=\"-1\">[<a class=\"miniselect\" href=\"viewDay.php?date=$this->date\">View Details</a>]&nbsp;[<a class=\"miniselect\" href=\"report.php?date=$this->date\">Generate Report</a>]</font></td></tr>";
    print "<tr><td colspan=\"2\">&nbsp;</td></tr>";
    $j=0;
    foreach ($distribution as $row) {
      if ($j%2==0) {
        print "<tr><td><b>$row->type</b>: $row->typecount</td>";
      }
      else {
        print "<td><b>$row->type</b>: $row->typecount</td></tr>";
      }
      $j++;
    }
    if ($j%2==1) print "<td>&nbsp;</td></tr>";

    print "</td></tr>\n";
    print "</table>\n";
  }
  
  #########################################################
  # PRIVATE FUNCTIONS
  #########################################################

  /////////////////////////////////////////////////////////
  // loadTasks()
  //
  // Takes:   Nothing
  // Returns: An array of tasks
  //
  // Creates a database connection and based off of the
  // date, returns an array of tasks
  //
  /////////////////////////////////////////////////////////
  private function loadTasks() {
    $this->db=new db(DB_HOST,DB_SCHEMA,DB_USER,DB_PASS);

    $sqloptions=implode(' OR ',$this->getSQLOptions());
    // $sql="SELECT * FROM tasks WHERE $sqloptions ORDER BY `schedule_start`";
    $sql="SELECT `task_id`, `filename`, `name`, `type`, `schedule_frequency`, `schedule_period`, `schedule_start`, `schedule_stop`, `schedule_wait-on` AS `schedule_wait_on`, `schedule_late-alert` AS `schedule_late_alert` FROM tasks WHERE $sqloptions ORDER BY `schedule_start`";

    $this->tasks=$this->db->dosql($sql);
  }
  
  /////////////////////////////////////////////////////////
  // getSQLOptions
  //
  // Takes:   Nothing
  // Returns: An array of SQL Options
  //
  // Depending on the class-defined date, this function
  // will return an array containing a list of options to
  // include in the WHERE clause of a SQL query.
  //
  /////////////////////////////////////////////////////////
  private function getSQLOptions() {
    $sqloptions=array();
    
    ### Set up options for SELECT statement.
    $sqloptions[0]='`schedule_frequency`="daily"';
    if ($this->day==1) {
      $sqloptions[count($sqloptions)]='`schedule_frequency`="1st-of-month"';
    }
    switch ($this->dow) {
    case SUNDAY:
      $sqloptions[count($sqloptions)]='`schedule_frequency`="sunday"';
      break;
    case MONDAY:
      $sqloptions[count($sqloptions)]='`schedule_frequency`="monday"';
      $sqloptions[count($sqloptions)]='`schedule_frequency`="mon-fri"';
      break;
    case TUESDAY:
      $sqloptions[count($sqloptions)]='`schedule_frequency`="tuesday"';
      $sqloptions[count($sqloptions)]='`schedule_frequency`="mon-fri"';
      $sqloptions[count($sqloptions)]='`schedule_frequency`="tues-sat"';
      break;
    case WEDNESDAY:
      $sqloptions[count($sqloptions)]='`schedule_frequency`="wednesday"';
      $sqloptions[count($sqloptions)]='`schedule_frequency`="mon-fri"';
      $sqloptions[count($sqloptions)]='`schedule_frequency`="tues-sat"';
      break;
    case THURSDAY:
      $sqloptions[count($sqloptions)]='`schedule_frequency`="thursday"';
      $sqloptions[count($sqloptions)]='`schedule_frequency`="mon-fri"';
      $sqloptions[count($sqloptions)]='`schedule_frequency`="tues-sat"';
      break;
    case FRIDAY:
      $sqloptions[count($sqloptions)]='`schedule_frequency`="friday"';
      $sqloptions[count($sqloptions)]='`schedule_frequency`="mon-fri"';
      $sqloptions[count($sqloptions)]='`schedule_frequency`="tues-sat"';
      break;
    case SATURDAY:
      $sqloptions[count($sqloptions)]='`schedule_frequency`="saturday"';
      $sqloptions[count($sqloptions)]='`schedule_frequency`="tues-sat"';
      break;
    }

    return $sqloptions;
  }
  
  #########################################################
  # STATIC FUNCTIONS
  #########################################################

  /////////////////////////////////////////////////////////
  // error()
  //
  // Takes:   Nothing
  // Returns: nothing
  //
  // Just tell the user that something went wrong.
  //
  /////////////////////////////////////////////////////////
  public static function error($errno) {
    print "There has been an error creating this (day) class <br>";
  }
}
?>
