<?php
///////////////////////////////////////////////////////////
// fileoutput.php.inc
///////////////////////////////////////////////////////////
//
// Abstract class to define necessary methodes for exorting
// data from the TaskMan calendaring system to a web
// browser.
//
// Version:    1.0
// Written by: Justin Zipkin 
//             justin.zipkin@db.com
//
// Modified:   15.11.2005 (jz)
//
///////////////////////////////////////////////////////////
abstract class fileoutput {
  protected $filename;
  protected $date, $day, $month, $year;
  protected $type;
  protected $tasks;
  protected $extension, $content_type;
  
  // Function to set the protected $extension variable
  abstract protected function setExtension();
  
  // function to set the protected $content_type variable
  abstract protected function setContentType();
  
  // Function to return a file to be ouput
  abstract protected function createFile();
  
  /////////////////////////////////////////////////////////
  // Constructor ($date)
  //
  // Takes:   A date
  // Returns: Nothing
  //
  // Populate some variables for use throughout the class
  // and check dependencies
  //
  /////////////////////////////////////////////////////////
  public function __construct($date) {
    if (!class_exists('day')) {
      include 'includes/header.php.inc';
      trigger_error("No day class available.",E_ERROR);
      print "<p class=\"error\">Error: There has been an internal server error, please contact a system administrator if this problem persists.</p>";
      include 'includes/footer.php.inc';
      exit(1);
    }

    $this->date=$date;
    $this->day = date('d', $this->date);
    $this->month = date('m', $this->date);
    $this->year = date('Y', $this->date);
    	
    // Instantiate a new day object for the selected date
		// and get a list of tasks
		$this->date=$date;
		$this->filename="Taskman_" . date("d-m-Y", $this->date);
		
		// Set the type of report to generate, used by some subclasses
		if (isset($_GET['type']))
		  $this->type=$_GET['type'];
		else
		  $this->type=0;
		
		$today=new day($date);
		$this->tasks=$today->getTasks();
		
		$this->setContentType();
		$this->setExtension();
  }

  /////////////////////////////////////////////////////////
  // outputFile ()
  //
  // Takes:   Nothing
  // Returns: Nothing
  //
  // Create and output a file to the web browser
  //
  /////////////////////////////////////////////////////////
  public function outputFile($content_type) {
    // Generate the file
    $file=$this->createFile();
    
    // Set the HTTP headers
    header("Content-type: " . $this->content_type);
    header("Content-disposition: attachment; filename=" . $this->filename . '.' . $this->extension);

    // And deliver the XML data
    print $file;
  }

  /////////////////////////////////////////////////////////
  // parseXML($input)
  //
  // Takes:   A string
  // Returns: A string
  //
  // Parses a string to be inserted into an XML document,
  // replacing special characters with the appropriate
  // representative format.
  //
  /////////////////////////////////////////////////////////
  final protected function parseXML($input) 
  {
    $output=$input;
    $output=preg_replace('/&/','&amp;',$output);   // &
    $output=preg_replace('/</','&lt;',$output);    // <
    $output=preg_replace('/>/','&gt;',$output);    // >
    $output=preg_replace('/"/','&quot;',$output);  // "
    $output=preg_replace('/\'/','&apos;',$output); // '
    
    return $output;
  }
}
?>
