<?php
///////////////////////////////////////////////////////////
// task-mysql.php.inc
///////////////////////////////////////////////////////////
//
// An implementation of the task class with MySQL syntax.
//
// Version:    1.2
// Written by: Justin Zipkin 
//             justin.zipkin@db.com
//
// Modified:   07.11.2005 (jz)
//             15.11.2005 (jz) - Got rid of getTaskArray
//                               and fixed loadTask
//             16.11.2005 (jz) - Added functions for
//                               getting task dependencies
//
///////////////////////////////////////////////////////////

class task {
  private $db;
  private $task_id, $filename;
  private $task;
  
  #########################################################
  # PUBLIC FUNCTIONS
  #########################################################
  
  /////////////////////////////////////////////////////////
  // Constructor ($task_id, $filename)
  //
  // Takes:   A task ID and filename as strings
  // Returns: Nothing
  //
  // Sets up a task object based off of:
  // 1) A task_id and filename
  //          OR
  // 2) If it is a special task, just the name of the task
  //
  /////////////////////////////////////////////////////////
  function __construct($task_id,$filename) {
    // If the user has passed in a task_id and filename
    // set up an object based off of that information
    if (!empty($filename)) {
      if (!class_exists('db')) {
        die('Error:  No database class included!  Please refer to documentation.');
      }
      
      $this->task_id=$task_id;
      $this->filename=$filename;
      $this->loadTask();
    }
    // If the user has not, then assume that the task_id
    // is actually a special task name.  We will create
    // a special instance of the objcet in that case.
    else {
      $this->task_id=0;
      $this->filename='';
      
      $this->task->{'task_id'}=0;
      $this->task->{'filename'}='';
      $this->task->{'name'}=$task_id;
      $this->task->{'type'}='Special Task';
      $this->task->{'schedule_frequency'}=0;
      $this->task->{'schedule_period'}=0;
      $this->task->{'schedule_start'}='00:00:00';
      $this->task->{'schedule_stop'}='23:59:00';
      $this->task->{'schedule_wait_on'}='';
      $this->task->{'schedule_late_alert'}='';
    }
  }
  
  /////////////////////////////////////////////////////////
  // getTasks()
  //
  // Takes:   Nothing
  // Returns: A task object
  //
  // This function will return a task object based on the
  // data in $this->task
  //
  /////////////////////////////////////////////////////////
  public function getTask() {
    return $this->task;
  }
  
  /////////////////////////////////////////////////////////
  // validTask()
  //
  // Takes:   Nothing
  // Returns: Boolean
  //
  // Returns whether or not the task has a valid name.
  //
  /////////////////////////////////////////////////////////
  public function validTask() {
    if (empty($this->task->name)) {
      return false;
    }
    else {
      return true;
    }
  }
  
  /////////////////////////////////////////////////////////
  // getDependencies($tasklist)
  //
  // Takes:   Nothing
  // Returns: An array of task data
  //
  // Obtains a list of predecessors and post(?)-decessors
  // for the current task and returns it to the caller.
  //
  /////////////////////////////////////////////////////////
  public function getDependencies() {
    $dependencies=array($this->task);
    $this->getPredecessors($dependencies);
    $this->getPostdecessors($dependencies);
    
    return $dependencies;
  }
  
  #########################################################
  # PRIVATE FUNCTIONS
  #########################################################
  
  /////////////////////////////////////////////////////////
  // loadTask()
  //
  // Takes:   Nothing
  // Returns: Nothing
  //
  // Creates a database connection and requests information
  // associated with a particular task.
  //
  /////////////////////////////////////////////////////////
  private function loadTask() {
    $this->db=new db(DB_HOST,DB_SCHEMA,DB_USER,DB_PASS);

    //$sql="SELECT `task_id`, `filename`, `name`, `type`, `schedule_frequency`, `schedule_period`, `schedule_start`, `schedule_stop`, `schedule_wait-on` AS `schedule_wait_on`, `schedule_late-alert` AS `schedule_late_alert` FROM tasks WHERE `task_id`='$this->task_id' ORDER BY `schedule_start` LIMIT 1";
    $sql="SELECT `task_id`, `filename`, `name`, `type`, `schedule_frequency`, `schedule_period`, `schedule_start`, `schedule_stop`, `schedule_wait-on` AS `schedule_wait_on`, `schedule_late-alert` AS `schedule_late_alert` FROM tasks WHERE `filename`='$this->filename' AND `task_id`='$this->task_id' ORDER BY `schedule_start` LIMIT 1";
    //$sql="SELECT * FROM tasks WHERE `filename`='$this->filename' AND `task_id`='$this->task_id' LIMIT 1";

    $dbresult=$this->db->dosql($sql);
    
    /*if (count($dbresult)<1) {
      $sql="SELECT `task_id`, `filename`, `name`, `type`, `schedule_frequency`, `schedule_period`, `schedule_start`, `schedule_stop`, `schedule_wait-on` AS `schedule_wait_on`, `schedule_late-alert` AS `schedule_late_alert` FROM tasks WHERE `task_id`='$this->task_id' ORDER BY `schedule_start` LIMIT 1";
      //$sql="SELECT * FROM tasks WHERE `filename`='$this->filename' AND `task_id`='$this->task_id' LIMIT 1";

      $dbresult=$this->db->dosql($sql);
    }*/
      
    
    $this->task=$dbresult[0];    
  }
    
  /////////////////////////////////////////////////////////
  // getPredecessors($tasklist)
  //
  // Takes:   An array of task data
  // Returns: Nothing
  //
  // Will push all of a tasks predecessors onto the passed
  // array
  //
  /////////////////////////////////////////////////////////
  private function getPredecessors(&$tasklist) {
    // Put the list of tasks to wait on in an array
    $namelist=explode(', ', $this->task->schedule_wait_on);
    
    foreach ($namelist as $curname) {
      if (!empty($curname)) {        
        // Firstly, if it is not a special task then we should treat
        // it normaly
        if (array_search($curname,$GLOBALS['SPECIALTASKS'])===FALSE) {
   
          // Look up the ID and filename of the task it waits on
          $curtask_id=task::lookupID($curname);
          $curfilename=task::lookupFilename($curtask_id,$curname);
          
          // Now that we have the ID and filename, we can create a task
          $curtask=new task($curtask_id,$curfilename);
          
          // Check to make sure that this task doesn't already exist
          // in our array, if it doesn't we need to add it
          if (array_search($curtask->getTask(),$tasklist)===FALSE) {
            
            // Push the task information onto the list
            array_push($tasklist, $curtask->getTask());
            
            // Now check if this new task waits on anything, if it does
            // we want to recursively call this function on that task
            if ($curtask->getTask()->schedule_wait_on!='') {
              //$tasklist=$curtask->getPredecessors($tasklist);
              $curtask->getPredecessors($tasklist);
            }
          }
        }
        else {
          // This is a special task
          // First thing we are going to do is create a new task
          // bsased off of the name.
          $curtask=new task($curname);
          
          // Only do something if this task isn't in the list already
          if (array_search($curtask->getTask(),$tasklist)===FALSE) {
  
            // Insert this task in the array as the second entry, just
            // under the task that the user wanted to look up.
            $tasklist=array_merge(
              array_slice($tasklist,0,1),
              array($curtask->getTask()),
              array_slice($tasklist,1));
          }
        }
      }
    }
  }
  
  /////////////////////////////////////////////////////////
  // getPostdecessors($tasklist)
  //
  // Takes:   An array of task data
  // Returns: Nothing
  //
  // Will push all of a tasks postdecessors onto the passed
  // array
  //
  /////////////////////////////////////////////////////////
  private function getPostdecessors(&$tasklist) {
    $db=new db(DB_HOST,DB_SCHEMA,DB_USER,DB_PASS);
    
    $sql="SELECT `task_id`, `filename`, `name`, `type`, `schedule_frequency`, `schedule_period`, `schedule_start`, `schedule_stop`, `schedule_wait-on` AS `schedule_wait_on`, `schedule_late-alert` AS `schedule_late_alert` FROM tasks WHERE `schedule_wait-on` LIKE '%". $this->task->name . "%'";

    // First we must get a list of all tasks that (we think) wait on this task
    $result=$db->dosql($sql);
    
    foreach ($result as $thisresult) {
      // Expand the list into an array
      $namelist=explode(', ', $thisresult->schedule_wait_on);
      
      foreach ($namelist as $curname) {
        // Because we used a LIKE %xx% type statement to determine tasks that wait
        // on this task, it may not be 100% accurate.  We are going to double check
        // right here to make sure that it is in fact waiting on this task and not
        // just a task that looks like this one.
        if ($curname==$this->task->name) {
          $curtask=new task($thisresult->task_id,$thisresult->filename);
          
          // Now we check if this task already exists somewhere in the array, if it
          // does we don't need to deal with it.
          if (array_search($curtask->getTask(),$tasklist)===FALSE) {
            
            // If it doesn't though, we are going to add the task to the list, and
            // also recursively call getPostdecessors on that task
            array_push($tasklist, $curtask->getTask());
            $curtask->getPostdecessors($tasklist);
          }
        }
      }
    }
  }
  
  #########################################################
  # STATIC FUNCTIONS
  #########################################################
  
  /////////////////////////////////////////////////////////
  // lookupID()
  //
  // Takes:   One or two strings
  // Returns: A string
  //
  // Lookup a TaskID based on a job name and filename
  //
  /////////////////////////////////////////////////////////
  public static function lookupID($name, $filename) {
    $db=new db(DB_HOST,DB_SCHEMA,DB_USER,DB_PASS);
    
    if (isset($filename)) {
      $sql="SELECT task_id FROM tasks WHERE `name`='$name' AND `filename`='$filename'";
      $result=$db->dosql($sql);
      if (empty($result)) {
        $sql="SELECT task_id FROM tasks WHERE `name`='$name' LIMIT 1";
        $result=$db->dosql($sql);
      }
    }
    else {
      $sql="SELECT task_id FROM tasks WHERE `name`='$name' LIMIT 1";
      $result=$db->dosql($sql);
    }
    
    //$result=$db->dosql($sql);
    
    return $result[0]->task_id;
  }

  /////////////////////////////////////////////////////////
  // lookupFilename()
  //
  // Takes:   Two strings
  // Returns: A string
  //
  // Lookup a filename based on a task id and name
  //
  /////////////////////////////////////////////////////////
  public static function lookupFilename($task_id, $name) {
    $db=new db(DB_HOST,DB_SCHEMA,DB_USER,DB_PASS);
    
    $sql="SELECT filename FROM tasks WHERE `task_id`='$task_id' AND `name`='$name' LIMIT 1";

    $result=$db->dosql($sql);
    
    return $result[0]->filename;
  }
  
  /////////////////////////////////////////////////////////
  // error()
  //
  // Takes:   Nothing
  // Returns: nothing
  //
  // Just tell the user that something went wrong.
  //
  /////////////////////////////////////////////////////////
  public static function error() {
    print "There has been an error creating this (task) class <br>";
  }
}
?>
